@@ -12,7 +12,7 @@
 from molSimplify.Classes.mol3D import *
 from molSimplify.Classes.atom3D import *
 from molSimplify.Scripts.geometry import * 
-
+from collections import Counter
 
 class ligand:
 	def __init__(self,master_mol,index_list,dent):
@@ -51,18 +51,20 @@ class ligand:
 					[new_active_set.append(element) for element in this_atoms_neighbors]
 				active_set = new_active_set
 		return trunc_mol
+
 def ligand_breakdown(mol):
 	# this function takes an octahedral 
 	# complex and returns ligands
 	metal_index = mol.findMetal()
-	bondedatoms = mol.getBondedAtoms(metal_index)
+	bondedatoms = mol.getBondedAtomsSmart(metal_index)
+	bonded_atom_symbols = [mol.getAtom(i).symbol() for i in bondedatoms]              
 	counter = 0
 	liglist = []
 	ligdents = []
 	ligcons = []
 	for atom in bondedatoms:
-		#print('this atom type is ' + mol.getAtom(atom).symbol())
-		#print('conection number ' + str(atom) + " of " + str(bondedatoms))
+		print('this atom type is ' + mol.getAtom(atom).symbol())
+		print('conection number ' + str(atom) + " of " + str(bondedatoms))
 		fragment = mol.findsubMol(atom,metal_index)
 		this_cons = [x for x in fragment if (x in bondedatoms)]
 		unique =  True
@@ -78,6 +80,7 @@ def ligand_breakdown(mol):
 			ligdents[matched] += 1
 	return liglist,ligdents,ligcons
 def ligand_assign(mol,liglist,ligdents,ligcons,loud=False,name=False):
+        valid = True
 	metal_index = mol.findMetal()
 	built_ligand_list  = list()
 	lig_natoms_list = list()
@@ -98,6 +101,13 @@ def ligand_assign(mol,liglist,ligdents,ligcons,loud=False,name=False):
 		print("max d = " + str(max_dent))
 		print("min_dent = " +  str(min_dent))
 		print("ligand list is" + str(liglist))
+		print('denticities are  ' + str(ligdents))
+	if max(ligdents) == 4 and min(ligdents) != 1:
+                valid = False
+                print('bad denticities: ' + str(ligdents)) 
+	if max(ligdents) >= 4:
+                valid = False
+                print('bad denticities: ' + str(ligdents)) 
 	eq_lig_list = list()
 	ax_lig_list = list()
 	ax_con_list = list()
@@ -167,11 +177,11 @@ def ligand_assign(mol,liglist,ligdents,ligcons,loud=False,name=False):
 				eq_con_list.append(ligcons[2])
 			elif min_dent == 2 and max_dent==2 and n_ligs ==3 and not n_unique_ligs ==1:
 				## this is a hetero/bidentate case
-				for i,ligs in enumerate(lig_list):
-					if lig_counts[i] == 2:
+				for i,ligs in enumerate(liglist):
+					if all_ligand_counts[i] == 2:
 						eq_lig_list.append(i)
 						eq_con_list.append(ligcons[i])
-					elif  lig_counts[i] == 1:
+					elif  all_ligand_counts[i] == 1:
 						ax_lig_list.append(i)
 						ax_con_list.append(ligcons[i])
 	elif (n_ligs == 6): # all mono  case, 
@@ -233,12 +243,13 @@ def ligand_assign(mol,liglist,ligdents,ligcons,loud=False,name=False):
 			#eq_lig=ligand_records[ligand_counts.index(4)]
 	ax_ligand_list = [built_ligand_list[i] for i in ax_lig_list]
 	eq_ligand_list = [built_ligand_list[i] for i in eq_lig_list]
-	if loud:
+	if loud and valid:
 		print('lig_nat_list',lig_natoms_list)
 		print('eq_liq is ind ',eq_lig_list)
 		print('ax_liq is ind ',ax_lig_list)
 		print('ax built lig [0] ext ind :' + str(built_ligand_list[ax_lig_list[0]].ext_int_dict.keys())) 
-		print('ax built lig [1] ext ind :' + str(built_ligand_list[ax_lig_list[1]].ext_int_dict.keys())) 
+                if len(ax_lig_list)>1:
+                        print('ax built lig [1] ext ind :' + str(built_ligand_list[ax_lig_list[1]].ext_int_dict.keys())) 
 		print('eq built lig [0] ext ind: '+str(built_ligand_list[eq_lig_list[0]].ext_int_dict.keys())) 
 		print('eq_con is '+str((eq_con_list)))
 		print('ax_con is '+str((ax_con_list)))
