@@ -11,12 +11,12 @@ from structgen import *
 from molSimplify.Scripts.io import *
 from molSimplify.Scripts.jobgen import *
 from molSimplify.Scripts.qcgen import *
+from molSimplify.Scripts.tsgen import *
 from molSimplify.Classes.rundiag import *
 import argparse, sys, os, shutil, itertools, random
 from collections import Counter
 from pkg_resources import resource_filename, Requirement
-
-import pybel
+import openbabel
 
 #######################################
 ### get subset between list1, list2 ###
@@ -209,7 +209,10 @@ def multigenruns(rundir,args,globs):
                     fname='N'+ch[1:]+'S'+sp
                 else:
                     fname='P'+ch+'S'+sp
-                emsg = rungen(rundir,args,fname,globs)
+                if args.tsgen:
+                    emsg = tsgen_supervisor(rundir,args,fname,globs)
+                else:
+                    emsg = rungen(rundir,args,fname,globs)
                 if emsg:
                     return emsg
     elif (multch):
@@ -221,7 +224,10 @@ def multigenruns(rundir,args,globs):
                 fname='N'+ch[1:]
             else:
                 fname='P'+ch[1:]
-            emsg = rungen(rundir,args,fname,globs)
+            if args.tsgen:
+                emsg = tsgen_supervisor(rundir,args,fname,globs)
+            else:
+                emsg = rungen(rundir,args,fname,globs)
             if emsg:
                 return emsg
     elif (multsp):
@@ -230,7 +236,10 @@ def multigenruns(rundir,args,globs):
         for sp in spins:
             args.spin = sp
             fname = 'S'+sp
-            emsg = rungen(rundir,args,fname,globs)
+            if args.tsgen:
+                emsg = tsgen_supervisor(rundir,args,fname,globs)
+            else:
+                emsg = rungen(rundir,args,fname,globs)
             if emsg:
                 return emsg
     else:
@@ -238,7 +247,10 @@ def multigenruns(rundir,args,globs):
             args.charge = args.charge[0]
         if args.spin:
             args.spin = args.spin[0]
-        emsg = rungen(rundir,args,fname,globs) # default
+        if args.tsgen:
+            emsg = tsgen_supervisor(rundir,args,fname,globs)
+        else:
+            emsg = rungen(rundir,args,fname,globs)
     return emsg
 
 #########################################################
@@ -251,34 +263,33 @@ def checkmultilig(ligs):
     # loop over ligands
     for i,lig in enumerate(ligs):
         connatoms = []
-        if ('.smi' in lig or '.xyz' in lig or '.mol' in lig):
-            lsuf = lig.split('.')[-1]
+        if '.smi' in lig:
             if '~' in lig:
                 lig = lig.replace('~',os.path.expanduser("~"))
             # read molecule
-            if glob.glob(lig):
-            	moll = list(pybel.readfile(lsuf,lig))
-                mols = [m.write('smi') for m in moll]
-                f = open(lig,'r')
+            flig = resource_filename(Requirement.parse("molSimplify"),"molSimplify/" +lig)
+            if glob.glob(flig):
+                f = open(flig,'r')
                 s = f.read().splitlines()
                 for ss in s:
-			ss = ss.replace('\t',' ')
-                	sf = filter(None,ss.split(' '))
-                        if len(sf) > 0:
-                            connatoms.append(sf[-1])
-                            multidx = i
-                        else:
-                            connatoms.append(False)
+                    ss = ss.replace('\t',' ')
+                    sf = filter(None,ss.split(' '))
+                    if len(sf) > 0:
+                        connatoms.append(sf[-1])
+                        multidx = i
+                    else:
+                        connatoms.append(False)
                 f.close()
-                if len(mols) > 1:
-                        mligs.append(mols)
+                if len(s) > 1:
+                    mligs.append(s)
                 else:
-                        mligs.append([lig])
+                    mligs.append([lig])
             else:
-            	mligs.append([lig])
+                mligs.append([lig])
         else:
             mligs.append([lig])
         tcats.append(connatoms)
+    print mligs
     ligandslist = list(itertools.product(*mligs))
     # convert tuple to list
     llist = []
@@ -291,6 +302,37 @@ def checkmultilig(ligs):
 
     return llist,tcats,multidx
 
+#################
+### draw mode ###
+#################
+def draw_supervisor(args,rundir):
+    if args.lig:
+        print('Due to technical limitations, we will draw only the first ligand.') 
+        print('To view multiple ligands at once, consider using the GUI instead.')
+        licores = getlicores()
+        l = args.lig[0]
+        lig, emsg = lig_load(l,licores)
+        lig.draw_svg(l)
+    elif args.core:
+        if len(args.core) > 1:
+            print('Due to technical limitations, we will draw only the first core.')
+        print('Drawing the core.')
+        if args.substrate:
+            print('Due to technical limitations, we can draw only one structure per run. To draw the substrate, run the program again.')
+        mcores = getmcores()
+        cc, emsg = core_load(args.core[0],mcores)
+        cc.draw_svg(args.core[0])
+    elif args.substrate:
+        if len(args.substrate) > 1:
+            print('Due to technical limitations, we will draw only the first substrate.')
+        print('Drawing the substrate.')
+        print args.substrate[0]
+        subcores = getsubcores()
+        substrate, emsg = substr_load(args.substrate[0],subcores)
+        substrate.draw_svg(args.substrate[0])
+    else:
+		print('You have not specified anything to draw. Currently supported: ligand, core, substrate')
+
 ##############################################
 ### normal structure generation of complex ###
 ##############################################
@@ -324,8 +366,6 @@ def rungen(rundir,args,chspfname,globs):
     smicat0 = [ss for ss in args.smicat] if args.smicat else False    
     # loop over ligands
     for mcount, mlig in enumerate(mligs):
-	if args.debug:
-        	print('in rungen, ligand is' + str(mlig) + ' of  ' + str(len(mligs)))
         args.smicat = [ss for ss in smicat0] if smicat0 else False
         args.checkdir, skip = False, False # initialize flags
         if len(mligs) > 0 and mligs[0]:
@@ -373,7 +413,8 @@ def rungen(rundir,args,chspfname,globs):
     ##### fetch smart name
         fname = name_complex(rundir,args.core,ligands,ligocc,mcount,args,bind=args.bind,bsmi=args.nambsmi)
         if globs.debug:
-		print('name is  '+fname)
+            print('fname is ' + str(fname))
+
         #        if args.bind:
                 # create folder for runs and check if it already exists
         #            if args.nambsmi:
@@ -541,9 +582,12 @@ def rungen(rundir,args,chspfname,globs):
                 fname = rootdir.rsplit('/',1)[-1]
                 shutil.move(rootdir,rundir+'/badjobs/'+fname)
             elif multidx != -1: # if ligand input was a list of smiles strings, write good smiles strings to separate list
-                f = open(ligfilename+'-good.smi','a')
-                f.write(args.lig[0])
-                f.close()  
+                try:
+                    f = open(ligfilename+'-good.smi','a')
+                    f.write(args.lig[0])
+                    f.close()
+                except:
+					0  
         elif not emsg:
             if args.gui:
                 qq = mQDialogInf('Folder skipped','Folder '+rootdir+' was skipped.')
@@ -552,3 +596,202 @@ def rungen(rundir,args,chspfname,globs):
                 print 'Folder '+rootdir+' was skipped..\n'
     return emsg    
 
+##############################################
+### transition state generation ###
+##############################################
+
+def tsgen_supervisor(rundir,args,chspfname,globs):
+	emsg = False
+	# load specified core into a mol3D object
+	mcores = getmcores()
+	cc, emsg = core_load(args.core,mcores)
+	if emsg:
+		return emsg	
+	cc.convert2mol3D()
+	subcores = getsubcores()
+	# load substrate molecule into a mol3D object
+	if len(args.substrate) > 1:
+		print('Currently only one substrate molecule is supported. Exiting...')
+		return
+	else:
+		substr, emsg = substr_load(args.substrate[0],subcores)
+	if emsg:
+		return emsg
+	substr.convert2mol3D()	
+	##### fetch smart name
+	fname = name_TS(rundir,args.core,substr,args,bind=args.bind,bsmi=args.nambsmi)
+	if globs.debug:
+		print('fname is ' + str(fname))
+	rootdir = fname
+	# check for charges/spin
+	rootcheck = False
+	if (chspfname):
+		rootcheck = rootdir
+		rootdir = rootdir + '/'+chspfname
+	if (args.suff):
+		rootdir += args.suff
+	# check for mannual overwrite of 
+	# directory name
+	if args.jobdir:
+		rootdir = rundir + args.jobdir
+	# check for top directory
+	skip = False
+	if  rootcheck and os.path.isdir(rootcheck) and not args.checkdirt and not skip:
+		args.checkdirt = True
+		if not args.rprompt:
+			flagdir=raw_input('\nDirectory '+rootcheck +' already exists. Keep both (k), replace (r) or skip (s) k/r/s: ')
+			if 'k' in flagdir.lower():
+				flagdir = 'keep'
+			elif 's' in flagdir.lower():
+					flagdir = 'skip'
+			else:
+				flagdir = 'replace'
+		else:
+			flagdir = 'replace'
+			# replace existing directory
+		if (flagdir=='replace'):
+			shutil.rmtree(rootcheck)
+			os.mkdir(rootcheck)
+		# skip existing directory
+		elif flagdir=='skip':
+			skip = True
+		# keep both (default)
+		else:
+			ifold = 1
+			while glob.glob(rootdir+'_'+str(ifold)):
+				ifold += 1
+				rootcheck += '_'+str(ifold)
+				os.mkdir(rootcheck)
+	elif rootcheck and (not os.path.isdir(rootcheck) or not args.checkdirt) and not skip:
+		if globs.debug:
+			print('rootcheck is  ' + str(rootcheck))
+		args.checkdirt = True
+		try:
+			os.mkdir(rootcheck)
+		except:
+			print 'Directory '+rootcheck+' can not be created. Exiting..\n'
+			return
+	# check for actual directory
+	if os.path.isdir(rootdir) and not args.checkdirb and not skip and not args.jobdir:
+		args.checkdirb = True
+		if not args.rprompt:
+			flagdir=raw_input('\nDirectory '+rootdir +' already exists. Keep both (k), replace (r) or skip (s) k/r/s: ')
+			if 'k' in flagdir.lower():
+				flagdir = 'keep'
+			elif 's' in flagdir.lower():
+					flagdir = 'skip'
+			else:
+				flagdir = 'replace'
+		else:
+			#qqb = qBoxFolder(args.gui.wmain,'Folder exists','Directory '+rootdir+' already exists. What do you want to do?')
+			#flagdir = qqb.getaction()
+			flagdir = 'replace'
+		# replace existing directory
+		if (flagdir=='replace'):
+			shutil.rmtree(rootdir)
+			os.mkdir(rootdir)
+		# skip existing directory
+		elif flagdir=='skip':
+			skip = True
+		# keep both (default)
+		else:
+			ifold = 1
+			while glob.glob(rootdir+'_'+str(ifold)):
+				ifold += 1
+			rootdir += '_'+str(ifold)
+			os.mkdir(rootdir)
+	elif not os.path.isdir(rootdir) or not args.checkdirb and not skip:
+		if not os.path.isdir(rootdir):
+			args.checkdirb = True
+			os.mkdir(rootdir)
+	####################################
+	############ GENERATION ############
+	####################################
+	# determine TS generation mode/reaction type
+	# 1: oxidative addition of a single group to an unsaturated complex (e.g., Fe(II) + O2 -> Fe(III)-O-O)
+	# 2: oxidative addition of two groups to an unsaturated complex (e.g., Pd + CH4 -> Pd(H)(CH3))
+	# 3: abstraction (ligand only reaction) (e.g., Fe(IV)=O + CH4 -> Fe(III)-OH + CH3)
+	# 1: compreact is the metal center, substreact is one atom
+	# 2: compreact is the metal center, substreact is two bonded atoms
+	# 3: compreact is not the metal center and bonded to only one atom, substreact is one atom
+	if len(args.compreact) == 1:
+		compreact = int(args.compreact[0]) - 1 # one-indexed in input
+	else:
+		print('Error: Currently only one complex reacting atom is supported. Exiting...')
+		return
+	if cc.getAtom(compreact).ismetal():
+		if len(args.substreact) == 1:
+			substreact = int(args.substreact[0]) - 1 # one-indexed in input
+			if len(substr.getBondedAtoms(substreact)) == 1:
+				if len(cc.getBondedAtomsOct(compreact)) < 6:
+					mode = 1
+					print('Mode 1: oxidative addition of a single group')
+				else:
+					print('Error: You have specified oxidative addition of a single group, but the metal atom is not unsaturated. Please check your input. Exiting...')
+					return
+			else:
+				print('Error: You have specified oxidative addition of a single group, but the substrate atom is not terminal. Please check your input. Exiting...')
+				return
+		elif len(args.substreact) == 2:
+			if int(args.substreact[1]) in substr.getBondedAtoms(int(args.substreact[0])):
+				if len(cc.getBondedAtomsOct(compreact)) < 5:
+					substreact = [int(i) - 1 for i in args.substreact] # one-indexed in input
+					mode = 2
+					print('Mode 2: oxidative addition of two groups')	
+				else:				
+					print('Error: You have specified oxidative addition of two groups, but the metal atom does not have two available empty sites. Please check your input. Exiting...')
+					return 
+			else:
+				print('Error: You have specified oxidative addition of two groups, but the two substrate groups are not bonded. Please check your input. Exiting...') 
+				return
+	elif len(cc.getBondedAtoms(compreact)) == 1:
+		if len(args.substreact) == 1:
+			substreact = int(args.substreact[0]) - 1 # one-indexed in input
+		else:
+			print('Error: You have specified abstraction, but specified more than one substrate atom. Please check your input. Exiting...')
+			return
+		if len(substr.getBondedAtoms(substreact)) == 1:
+			mode = 3
+			print('Mode 3: Abstraction')
+		else:
+			print('Error: You have specified abstraction, but the substrate atom is not terminal. Please check your input. Exiting...')
+			return
+	else:
+		print('Error: You have specified abstraction, but the abstracting atom is not terminal. Please check your input. Exiting...')
+		return
+	if not skip:
+		# generate xyz files
+		strfiles,emsg,this_diag = tsgen(mode,args,rootdir,cc,substr,compreact,substreact,globs)
+		# generate QC input files
+		if args.qccode and not emsg:
+			args.runtyp = 'ts'
+			if args.charge and (isinstance(args.charge, list)):
+				args.charge = args.charge[0]
+			if args.spin and (isinstance(args.spin, list)):
+				args.spin = args.spin[0]
+			if args.qccode.lower() in 'terachem tc Terachem TeraChem TERACHEM TC':
+				jobdirs = multitcgen(args,strfiles)
+				print 'TeraChem input files generated!'
+			elif 'gam' in args.qccode.lower():
+				jobdirs = multigamgen(args,strfiles)
+				print 'GAMESS input files generated!'
+			elif 'qch' in args.qccode.lower():
+				jobdirs = multiqgen(args,strfiles)
+				print 'QChem input files generated!'
+			else:
+				print 'Only TeraChem, GAMESS and QChem are supported right now.\n'
+		# generate jobscripts
+		if args.jsched and not emsg:
+			if args.jsched in 'SBATCH SLURM slurm sbatch':
+				slurmjobgen(args,jobdirs)
+				print 'SLURM jobscripts generated!'
+			elif args.jsched in 'SGE Sungrid sge':
+				sgejobgen(args,jobdirs)
+				print 'SGE jobscripts generated!'
+		if this_diag.sanity: # move to separate subdirectory if generated structure was bad
+			fname = rootdir.rsplit('/',1)[-1]
+			shutil.move(rootdir,rundir+'/badjobs/'+fname)
+	elif not emsg:
+		print 'Folder '+rootdir+' was skipped..\n'
+	return emsg  
+	
